# Copyright (c) 2013, Fafadia Tech and contributors
# For license information, please see license.txt

"""
Report logic and description is available on following youtube link

https://www.youtube.com/watch?v=A_2hcSehHE0&feature=youtu.be
"""

from __future__ import unicode_literals
import frappe
from frappe import _


def get_customer_rfq(opportunities):
    """
    This method is used to get quotation count for given opportunities where status is not equal to Submitted and even if there is no Quotation
    Input:
        opportunities name (List)
    Output:
        open_customer_rfq (count) (int)
    """
    open_customer_rfq = 0
    for o in opportunities:
        q = frappe.db.count("Quotation", filters={
                "opportunity": ["=", o.name],
                "status": ["!=", "Submitted"]
            })
        if q > 0:
            open_customer_rfq += 1
        if not q:
            open_customer_rfq += 1
    return open_customer_rfq


def get_opportunties(assigned_to, quote_status, status_comp):
    """
    This method is used to get filtered list of opportunities assigned to specific procurement user and
    given qoute status and status comparision i.e (= 'Equal To', != 'Not Equal To')
    Input:
        assigned_to (user name) (string) format: '["user@example.com"]'
        quote_status (string)
        status_comp (string) (example: = and !=)
    Output:
        opportunities (list) (list containing opportunities name)
    """
    opportunities = frappe.get_list("Opportunity", filters={
        "_assign": assigned_to,
        "gtl_quote_status": [status_comp, quote_status]})
    return opportunities


def execute(filters=None):
        """
        GTL Procurement Report

        Report Logic and Description is available on following youtube link

        https://www.youtube.com/watch?v=A_2hcSehHE0&feature=youtu.be
        """
        data = []
        columns = [{"fieldname": "user", "label": _("User"), "fieldtype": "Data"},
                {"fieldname": "open_customer_rfq", "label": _("Open Customer RFQ"), "fieldtype": "Data"},
                {"fieldname": "rfq_not_floated", "label": _("RFQ Not Floated"), "fieldtype": "Data"},
                {"fieldname": "rfq_floated", "label": _("RFQ Floated"), "fieldtype": "Data"},
                {"fieldname": "end_user_input_awaited", "label": _("End User Input Awaited"), "fieldtype": "Data"},
                {"fieldname": "quote_pending", "label": _("Quote Pending"), "fieldtype": "Data"},
                {"fieldname": "quote_received", "label": _("Quote Received"), "fieldtype": "Data"},
                {"fieldname": "clarification_pending", "label": _("Clarification Pending"), "fieldtype": "Data"},
                {"fieldname": "bid_preperation_pending", "label": _("Bid Preperation Pending"), "fieldtype": "Data"},
                {"fieldname": "bid_submitted_to_tl", "label": _("Bid Submitted to TL"), "fieldtype": "Data"},
                {"fieldname": "bid_approved_by_tl", "label": _("Bid Approved by TL"), "fieldtype": "Data"},
                {"fieldname": "bid_rejected_by_tl", "label": _("Bid Rejected by TL"), "fieldtype": "Data"},
                ]
        # get list of all users with role "Procurement Engineer"
        users = frappe.get_list("User", filters={
            "role_profile_name": "V1 Procurement Engineer"},
            fields=["name", "full_name"])

        for u in users:
            data_dict = {}
            data_dict["user"] = u.full_name
            assigned_to = '["{}"]'.format(u.name)
            # get list of opportunities assigned to specific user
            opportunities = frappe.get_list("Opportunity", {"_assign": assigned_to})
            # get open customerrfq count defined in above function
            open_customer_rfq = get_customer_rfq(opportunities)
            data_dict["open_customer_rfq"] = open_customer_rfq

            opportunities = get_opportunties(
                assigned_to, "RFQ Not Floated", "=")
            rfq_not_floated = get_customer_rfq(opportunities)
            data_dict["rfq_not_floated"] = rfq_not_floated

            opportunities = get_opportunties(
                assigned_to, "RFQ Not Floated", "!=")
            rfq_floated = get_customer_rfq(opportunities)
            data_dict["rfq_floated"] = rfq_floated

            opportunities = get_opportunties(
                assigned_to, "End-User Input Awaited", "=")
            end_user_input_awaited = get_customer_rfq(opportunities)
            data_dict["end_user_input_awaited"] = end_user_input_awaited

            opportunities = get_opportunties(
                assigned_to, "Quote Pending", "=")
            quote_pending = get_customer_rfq(opportunities)
            data_dict["quote_pending"] = quote_pending

            opportunities = get_opportunties(
                assigned_to, "Clarifications-Technical ", "=")
            ct = get_customer_rfq(opportunities)

            opportunities = get_opportunties(
                assigned_to, "Clarifications-Commercial", "=")
            cc = get_customer_rfq(opportunities)

            clarification_pending = ct + cc
            data_dict["clarification_pending"] = clarification_pending

            opportunities = get_opportunties(
                assigned_to, "Bid Preparation Pending", "=")
            bid_preperation_pending = get_customer_rfq(opportunities)
            data_dict["bid_preperation_pending"] = bid_preperation_pending

            opportunities = get_opportunties(
                assigned_to, "Bid Submitted to TL", "=")
            bid_submitted_to_tl = get_customer_rfq(opportunities)
            data_dict["bid_submitted_to_tl"] = bid_submitted_to_tl

            opportunities = get_opportunties(
                assigned_to, "Bid Approved by TL", "=")
            bid_approved_by_tl = get_customer_rfq(opportunities)
            data_dict["bid_approved_by_tl"] = bid_approved_by_tl

            opportunities = get_opportunties(
                assigned_to, "Bid Rejected by TL", "=")
            bid_rejected_by_tl = get_customer_rfq(opportunities)
            data_dict["bid_rejected_by_tl"] = bid_rejected_by_tl

            data_dict["quote_received"] = clarification_pending + \
                bid_preperation_pending + bid_submitted_to_tl + \
                bid_approved_by_tl + bid_rejected_by_tl

            data.append(data_dict)

        return columns, data

